@@ -1,5 +1,5 @@
 /* audispd-config.c -- 
- * Copyright 2007 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2007-08 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -31,6 +31,7 @@
 #include <stdlib.h>
 #include <libgen.h>
 #include <ctype.h>
+#include <limits.h>
 #include "audispd-config.h"
 #include "private.h"
 
@@ -66,6 +67,8 @@ static int name_parser(struct nv_pair *nv, int line,
 		daemon_conf_t *config);
 static int overflow_action_parser(struct nv_pair *nv, int line, 
 		daemon_conf_t *config);
+static int priority_boost_parser(struct nv_pair *nv, int line,
+		daemon_conf_t *config);
 static int sanity_check(daemon_conf_t *config, const char *file);
 
 static const struct kw_pair keywords[] = 
@@ -73,7 +76,8 @@ static const struct kw_pair keywords[] =
   {"q_depth",                  q_depth_parser,			0 },
   {"name_format",              name_format_parser,		0 },
   {"name",                     name_parser,			0 },
-  {"overflow_action",          overflow_action_parser,			0 },
+  {"overflow_action",          overflow_action_parser,		0 },
+  {"priority_boost",           priority_boost_parser,           0 },
   { NULL,                      NULL }
 };
 
@@ -102,8 +106,9 @@ static const struct nv_list overflow_actions[] =
 */
 void clear_config(daemon_conf_t *config)
 {
-	config->q_depth = 64;
+	config->q_depth = 80;
 	config->overflow_action = O_SYSLOG;
+	config->priority_boost = 4;
 	config->node_name_format = N_NONE;
 	config->name = NULL;
 }
@@ -378,6 +383,44 @@ static int overflow_action_parser(struct nv_pair *nv, int line,
         return 1;
 }
 
+static int priority_boost_parser(struct nv_pair *nv, int line,
+	struct daemon_conf *config)
+{
+	const char *ptr = nv->value;
+	int i;
+
+	audit_msg(LOG_DEBUG, "priority_boost_parser called with: %s",
+       				nv->value);
+
+	/* check that all chars are numbers */
+	for (i=0; ptr[i]; i++) {
+		if (!isdigit(ptr[i])) {
+			audit_msg(LOG_ERR,
+				"Value %s should only be numbers - line %d",
+				nv->value, line);
+			return 1;
+		}
+	}
+	/* convert to unsigned int */
+	errno = 0;
+	i = strtoul(nv->value, NULL, 10);
+	if (errno) {
+		audit_msg(LOG_ERR,
+			"Error converting string to a number (%s) - line %d",
+			strerror(errno), line);
+		return 1;
+	}
+	/* Check its range */
+	if (i > INT_MAX) {
+		audit_msg(LOG_ERR,
+			"Error - converted number (%s) is too large - line %d",
+			nv->value, line);
+		return 1;
+	}
+	config->priority_boost = (unsigned int)i;
+	return 0;
+}
+
 /*
  * This function is where we do the integrated check of the audispd config
  * options. At this point, all fields have been read. Returns 0 if no
