@@ -1,5 +1,5 @@
 /* audispd.c --
- * Copyright 2007 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2007-08 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -31,7 +31,6 @@
 #include <pthread.h>
 #include <dirent.h>
 #include <fcntl.h>
-#include <pthread.h>
 #include <sys/poll.h>
 #include <netdb.h>
 #include <arpa/inet.h>
@@ -308,16 +307,33 @@ int main(int argc, char *argv[])
 	sigaction(SIGCHLD, &sa, NULL);
 
 	/* move stdin to its own fd */
-	audit_fd = dup(0);
-	fcntl(audit_fd, F_SETFD, FD_CLOEXEC);
+	if (argc == 3 && strcmp(argv[1], "--input") == 0) 
+		audit_fd = open(argv[2], O_RDONLY);
+	else
+		audit_fd = dup(0);
+	if (audit_fd < 0) {
+		syslog(LOG_ERR, "Failed setting up input, exiting");
+		return 1;
+	}
 
 	/* Make all descriptors point to dev null */
 	i = open("/dev/null", O_RDWR);
 	if (i >= 0) {
-		dup2(0, i);
-		dup2(1, i);
-		dup2(2, i);
+		if (dup2(0, i) < 0 || dup2(1, i) < 0 || dup2(2, i) < 0) {
+			syslog(LOG_ERR, "Failed duping /dev/null %s, exiting",
+					strerror(errno));
+			return 1;
+		}
 		close(i);
+	} else {
+		syslog(LOG_ERR, "Failed opening /dev/null %s, exiting",
+					strerror(errno));
+		return 1;
+	}
+	if (fcntl(audit_fd, F_SETFD, FD_CLOEXEC) < 0) {
+		syslog(LOG_ERR, "Failed protecting input %s, exiting",
+					strerror(errno));
+		return 1;
 	}
 
 	/* init the daemon's config */
@@ -332,8 +348,20 @@ int main(int argc, char *argv[])
 		return 0;
 	}
 
+	/* Plugins are started with the auditd priority */
 	i = start_plugins(&plugin_conf);
 
+	/* Now boost priority to make sure we are getting time slices */
+	if (daemon_config.priority_boost != 0) {
+		errno = 0;
+		(void) nice((int)-daemon_config.priority_boost);
+		if (errno) {
+			syslog(LOG_ERR, "Cannot change priority (%s)",
+					strerror(errno));
+			/* Stay alive as this is better than stopping */
+		}
+	}
+
 	/* Let the queue initialize */
 	init_queue(daemon_config.q_depth);
 	syslog(LOG_NOTICE, 
@@ -405,8 +433,8 @@ static int safe_exec(plugin_conf_t *conf)
 
 	/* Set up comm with child */
 	dup2(conf->plug_pipe[0], 0);
-	close(conf->plug_pipe[0]);
-	close(conf->plug_pipe[1]);
+	for (i=3; i<24; i++)	 /* Arbitrary number */
+		close(i);
 
 	/* Child */
 	argv[0] = (char *)conf->path;
@@ -681,29 +709,51 @@ static void *inbound_thread_main(void *arg)
 static void process_inbound_event(int fd)
 {
 	int rc;
-	struct iovec vec[2];
+	struct iovec vec;
 	event_t *e = malloc(sizeof(event_t));
 	if (e == NULL) 
 		return;
 	memset(e, 0, sizeof(event_t));
 
 	/* Get header first. It is fixed size */
-	vec[0].iov_base = &e->hdr;
-	vec[0].iov_len = sizeof(struct audit_dispatcher_header);
+	vec.iov_base = &e->hdr;
+	vec.iov_len = sizeof(struct audit_dispatcher_header);
 	do {
-		rc = readv(fd, &vec[0], 1);
+		rc = readv(fd, &vec, 1);
 	} while (rc < 0 && errno == EINTR);
 
+	if (rc <= 0) {
+		if (rc == 0)
+			stop = 1; // End of File
+		free(e);
+		return;
+	}
+
 	if (rc > 0) {
+		/* Sanity check */
+		if (e->hdr.ver != AUDISP_PROTOCOL_VER ||
+				e->hdr.hlen != sizeof(e->hdr) ||
+				e->hdr.size > MAX_AUDIT_MESSAGE_LENGTH) {
+			free(e);
+			syslog(LOG_ERR,
+				"Dispatcher protocol mismatch, exiting");
+			exit(1);
+		}
+
 		/* Next payload */
-		vec[1].iov_base = &e->data;
-		vec[1].iov_len = e->hdr.size;
+		vec.iov_base = e->data;
+		vec.iov_len = e->hdr.size;
 		do {
-			rc = readv(fd, &vec[1], 1);
+			rc = readv(fd, &vec, 1);
 		} while (rc < 0 && errno == EINTR);
 
 		if (rc > 0)
 			enqueue(e, &daemon_config);
+		else {
+			if (rc == 0)
+				stop = 1; // End of File
+			free(e);
+		}
 	}
 }
 
