@@ -117,13 +117,16 @@ void enqueue(event_t *e, struct daemon_conf *config)
 {
 	unsigned int n, retry_cnt = 0;
 
-	if (processing_suspended)
+	if (processing_suspended) {
+		free(e);
 		return;
+	}
 
 retry:
 	// We allow 3 retries and then its over
 	if (retry_cnt > 3) {
 		do_overflow_action(config);
+		free(e);
 		return;
 	}
 	pthread_mutex_lock(&queue_lock);
