@@ -37,21 +37,17 @@ typedef enum { AUSOURCE_LOGS, AUSOURCE_FILE, AUSOURCE_FILE_ARRAY,
 	AUSOURCE_BUFFER, AUSOURCE_BUFFER_ARRAY,
 	AUSOURCE_DESCRIPTOR, AUSOURCE_FILE_POINTER, AUSOURCE_FEED } ausource_t;
 
-/* This defines the types of searches that can be done */
+/* This used to define the types of searches that can be done.  It is not used
+   any more. */
 typedef enum {
 	AUSEARCH_UNSET,
 	AUSEARCH_EXISTS,
 	AUSEARCH_EQUAL, AUSEARCH_NOT_EQUAL,
 	AUSEARCH_TIME_LT, AUSEARCH_TIME_LE, AUSEARCH_TIME_GE, AUSEARCH_TIME_GT,
 	AUSEARCH_TIME_EQ,
-	/* When comparing, use the value that would be returned by
-	   auparse_interpret_field() instead of the raw value. */
 	AUSEARCH_INTERPRETED = 0x40000000
 } ausearch_op_t;
 
-#define AUSEARCH_OP_IS_TIME__(OP) \
-  ((OP) >= AUSEARCH_TIME_LT && (OP) <= AUSEARCH_TIME_EQ)
-
 /* This determines where to position the cursor when a search completes */
 typedef enum { AUSEARCH_STOP_EVENT, AUSEARCH_STOP_RECORD,
 	AUSEARCH_STOP_FIELD } austop_t;
