@@ -1,5 +1,5 @@
 /* auparse.c --
- * Copyright 2006-07 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2006-08 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -21,6 +21,7 @@
  */
 
 #include "config.h"
+#include "expression.h"
 #include "internal.h"
 #include "auparse.h"
 #include "interpret.h"
@@ -195,10 +196,10 @@ auparse_state_t *auparse_init(ausource_t source, const void *b)
 	au->line_pushed = 0;
 	aup_list_create(&au->le);
 	au->parse_state = EVENT_EMPTY;
-	aurule_create(&au->rules);
+	au->expr = NULL;
 	au->find_field = NULL;
 	au->search_where = AUSEARCH_STOP_EVENT;
-	au->search_how = AUSEARCH_RULE_CLEAR;
+	au->regex_valid = 0;
 
 	return au;
 bad_exit:
@@ -247,8 +248,8 @@ static void consume_feed(auparse_state_t *au, int flush)
 
 int auparse_feed(auparse_state_t *au, const char *data, size_t data_len)
 {
-	if (databuf_append(&au->databuf, data, data_len) < 0) return -1;
-	//if (debug) databuf_print(&au->databuf, 1, "auparse_feed() added %d bytes", data_len);
+	if (databuf_append(&au->databuf, data, data_len) < 0)
+		return -1;
 	consume_feed(au, 0);
 	return 0;
 }
@@ -297,67 +298,75 @@ int auparse_reset(auparse_state_t *au)
 }
 
 
-static struct nv_pair optab[] = {
-	{AUSEARCH_EXISTS, "exists"},
-	{AUSEARCH_EQUAL, "="},
-	{AUSEARCH_NOT_EQUAL, "!="},
-};
-#define OP_NAMES (sizeof(optab)/sizeof(optab[0]))
-
-static int lookup_op(const char *op)
+/* Add EXPR to AU, using HOW to select the combining operator.
+   On success, return 0.
+   On error, free EXPR set errno and return -1.
+   NOTE: EXPR is freed on error! */
+static int add_expr(auparse_state_t *au, struct expr *expr, ausearch_rule_t how)
 {
-        int i;
+	if (au->expr == NULL)
+		au->expr = expr;
+	else if (how == AUSEARCH_RULE_CLEAR) {
+		expr_free(au->expr);
+		au->expr = expr;
+	} else {
+		struct expr *e;
 
-        for (i = 0; i < OP_NAMES; i++)
-                if (strcmp(optab[i].name, op) == 0)
-                        return optab[i].value;
+		e = expr_create_binary(how == AUSEARCH_RULE_OR ? EO_OR : EO_AND,
+				       au->expr, expr);
+		if (e == NULL) {
+			int err;
 
-        return -1;
+			err = errno;
+			expr_free(expr);
+			errno = err;
+			return -1;
+		}
+		au->expr = e;
+	}
+	return 0;
 }
 
-
 static int ausearch_add_item_internal(auparse_state_t *au, const char *field,
-	const char *op, const char *value, ausearch_rule_t how,
-	ausearch_op_t op_flag)
+	const char *op, const char *value, ausearch_rule_t how, unsigned op_eq,
+	unsigned op_ne)
 {
-	aurulenode rn;
-	int t_op = lookup_op(op);
-	if (t_op < 0)
-		goto err_out;
+	struct expr *expr;
 
 	// Make sure there's a field
 	if (field == NULL)
 		goto err_out;
 
 	// Do not allow regex to get replaced this way
-	if (au->search_how == AUSEARCH_RULE_REGEX)
+	if (au->regex_valid != 0)
 		goto err_out;
 
 	// Make sure how is within range
 	if (how < AUSEARCH_RULE_CLEAR || how > AUSEARCH_RULE_AND)
 		goto err_out;
-	if (how == AUSEARCH_RULE_CLEAR) {
-		aurule_clear(&au->rules);
-	} else if (how != au->search_how && 
-				au->search_how != AUSEARCH_RULE_CLEAR) {
-		errno = EEXIST;
-		return -1;
-	}
 
 	// All pre-checks are done, build a rule
-	rn.search_op = t_op | op_flag;
-	if (t_op == AUSEARCH_EXISTS)
-		rn.v.field.val = NULL;
+	if (strcmp(op, "exists") == 0)
+		expr = expr_create_field_exists(field);
 	else {
+		unsigned t_op;
+
+		if (strcmp(op, "=") == 0)
+			t_op = op_eq;
+		else if (strcmp(op, "!=") == 0)
+			t_op = op_ne;
+		else
+			goto err_out;
 		if (value == NULL)
 			goto err_out;
-		rn.v.field.val = strdup(value);
+		expr = expr_create_comparison(field, t_op, value);
 	}
-	rn.v.field.field = strdup(field);
-	aurule_append(&au->rules, &rn);
-	au->search_how = how;
-
+	if (expr == NULL)
+		return -1;
+	if (add_expr(au, expr, how) != 0)
+		return -1; /* expr is freed by add_expr() */
 	return 0;
+
 err_out:
 	errno = EINVAL;
 	return -1;
@@ -366,83 +375,104 @@ err_out:
 int ausearch_add_item(auparse_state_t *au, const char *field, const char *op,
 	const char *value, ausearch_rule_t how)
 {
-	return ausearch_add_item_internal(au, field, op, value, how, 0);
+	return ausearch_add_item_internal(au, field, op, value, how, EO_RAW_EQ,
+					  EO_RAW_NE);
 }
 
 int ausearch_add_interpreted_item(auparse_state_t *au, const char *field,
 	const char *op, const char *value, ausearch_rule_t how)
 {
 	return ausearch_add_item_internal(au, field, op, value, how,
-					  AUSEARCH_INTERPRETED);
-}
-
-static int lookup_ts_op(const char *op)
-{
-	static const struct nv_pair ts_tab[] = {
-		{AUSEARCH_TIME_LT, "<"},
-		{AUSEARCH_TIME_LE, "<="},
-		{AUSEARCH_TIME_GE, ">="},
-		{AUSEARCH_TIME_GT, ">"},
-		{AUSEARCH_TIME_EQ, "="},
-	};
-
-        size_t i;
-
-        for (i = 0; i < sizeof(ts_tab) / sizeof(*ts_tab); i++)
-                if (strcmp(ts_tab[i].name, op) == 0)
-                        return ts_tab[i].value;
-
-        return -1;
+					  EO_INTERPRETED_EQ, EO_INTERPRETED_NE);
 }
 
 int ausearch_add_timestamp_item(auparse_state_t *au, const char *op, time_t sec,
 				unsigned milli, ausearch_rule_t how)
 {
-	aurulenode rn;
-	int t_op = lookup_ts_op(op);
-	if (op < 0)
-		goto err_out;
+	static const struct {
+		unsigned value;
+		char name[3];
+	} ts_tab[] = {
+		{EO_VALUE_LT, "<"},
+		{EO_VALUE_LE, "<="},
+		{EO_VALUE_GE, ">="},
+		{EO_VALUE_GT, ">"},
+		{EO_VALUE_EQ, "="},
+	};
+
+	struct expr *expr;
+        size_t i;
+	unsigned t_op;
+
+        for (i = 0; i < sizeof(ts_tab) / sizeof(*ts_tab); i++) {
+                if (strcmp(ts_tab[i].name, op) == 0)
+			goto found_op;
+	}
+	goto err_out;
+found_op:
+	t_op = ts_tab[i].value;
+
 	if (milli >= 1000)
 		goto err_out;
 
 	// Do not allow regex to get replaced this way
-	if (au->search_how == AUSEARCH_RULE_REGEX)
+	if (au->regex_valid != 0)
 		goto err_out;
 
 	// Make sure how is within range
 	if (how < AUSEARCH_RULE_CLEAR || how > AUSEARCH_RULE_AND)
 		goto err_out;
-	if (how == AUSEARCH_RULE_CLEAR) {
-		aurule_clear(&au->rules);
-	} else if (how != au->search_how
-		   && au->search_how != AUSEARCH_RULE_CLEAR) {
-		errno = EEXIST;
-		return -1;
-	}
 
 	// All pre-checks are done, build a rule
-	rn.search_op = t_op;
-	rn.v.timestamp.sec = sec;
-	rn.v.timestamp.milli = milli;
-	aurule_append(&au->rules, &rn);
-	au->search_how = how;
-
+	expr = expr_create_timestamp_comparison(t_op, sec, milli);
+	if (expr == NULL)
+		return -1;
+	if (add_expr(au, expr, how) != 0)
+		return -1; /* expr is freed by add_expr() */
 	return 0;
+
 err_out:
 	errno = EINVAL;
 	return -1;
 }
 
+int ausearch_add_expression(auparse_state_t *au, const char *expression,
+			    char **error, ausearch_rule_t how)
+{
+	struct expr *expr;
+
+	// Do not allow regex to get replaced this way
+	if (au->regex_valid != 0)
+		goto err_einval;
+	if (how < AUSEARCH_RULE_CLEAR || how > AUSEARCH_RULE_AND)
+		goto err_einval;
+
+	expr = expr_parse(expression, error);
+	if (expr == NULL) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	if (add_expr(au, expr, how) != 0)
+		goto err; /* expr is freed by add_expr() */
+	return 0;
+
+err_einval:
+	errno = EINVAL;
+err:
+	*error = NULL;
+	return -1;
+}
+
 int ausearch_add_regex(auparse_state_t *au, const char *expr)
 {
-	aurulenode rn;
 	int rc;
 
 	// Make sure there's an expression
 	if (expr == NULL)
 		goto err_out;
 
-	if (au->search_how != AUSEARCH_RULE_CLEAR)
+	if (au->regex_valid != 0 || au->expr != NULL)
 		goto err_out;
 
 	// Compile expression now to make sure the expression is correct
@@ -450,13 +480,7 @@ int ausearch_add_regex(auparse_state_t *au, const char *expr)
 	if (rc) {
 		goto err_out;
 	}
-
-	// Store it away just in case.
-	rn.search_op = AUSEARCH_UNSET;
-	rn.v.field.field = strdup(expr);
-	rn.v.field.val = NULL;
-	aurule_append(&au->rules, &rn);
-	au->search_how = AUSEARCH_RULE_REGEX;
+	au->regex_valid = 1;
 
 	return 0; 
 
@@ -478,12 +502,17 @@ int ausearch_set_stop(auparse_state_t *au, austop_t where)
 
 void ausearch_clear(auparse_state_t *au)
 {
-	if (au->search_how == AUSEARCH_RULE_REGEX && au->rules.cnt > 0)
+	if (au->regex_valid != 0) {
 		regfree(&au->regex);
-	aurule_clear(&au->rules);
+		au->regex_valid = 0;
+	}
+	if (au->expr != NULL) {
+		expr_free(au->expr);
+		au->expr = NULL;
+	}
 	au->search_where = AUSEARCH_STOP_EVENT;
-	au->search_how = AUSEARCH_RULE_CLEAR;
 }
+hidden_def(ausearch_clear)
 
 void auparse_destroy(auparse_state_t *au)
 {
@@ -855,75 +884,15 @@ static int ausearch_reposition_cursors(auparse_state_t *au)
 	return rc;
 }
 
-/* Return the result of interpreting rule on the event record r. */
-static int ausearch_interpret_rule(auparse_state_t *au, rnode *r,
-				   aurulenode *rule)
-{
-	int search_op;
-
-	search_op = (rule->search_op & ~AUSEARCH_INTERPRETED);
-	if (AUSEARCH_OP_IS_TIME__(search_op)) {
-		int val;
-
-		if (au->le.e.sec < rule->v.timestamp.sec)
-			val = -1;
-		else if (au->le.e.sec > rule->v.timestamp.sec)
-			val = 1;
-		else if (au->le.e.milli < rule->v.timestamp.milli)
-			val = -1;
-		else if (au->le.e.milli > rule->v.timestamp.milli)
-			val = 1;
-		else
-			val = 0;
-		switch (search_op) {
-		case AUSEARCH_TIME_LT:
-			return val < 0;
-		case AUSEARCH_TIME_LE:
-			return val <= 0;
-		case AUSEARCH_TIME_GE:
-			return val >= 0;
-		case AUSEARCH_TIME_GT:
-			return val > 0;
-		case AUSEARCH_TIME_EQ:
-			return val == 0;
-		default:
-			abort();
-		}
-	} else {
-		int found, rc;
-		const char *val;
-
-		found = nvlist_find_name(&r->nv, rule->v.field.field);
-		if (!found)
-			return 0;
-		if (search_op == AUSEARCH_EXISTS)
-			return 1;
-
-		if ((rule->search_op & AUSEARCH_INTERPRETED) != 0)
-			val = nvlist_interp_cur_val(r);
-		else
-			val = nvlist_get_cur_val(&r->nv);
-		rc = strcmp(rule->v.field.val, val);
-		if (search_op == AUSEARCH_EQUAL)
-			return rc == 0;
-		else
-			return rc != 0;
-	}
-}
-
 /* This is called during search once per each record. It walks the list
  * of nvpairs and decides if a field matches. */
 static int ausearch_compare(auparse_state_t *au)
 {
 	rnode *r;
 
-	if (au->search_how == AUSEARCH_RULE_REGEX) {
+	if (au->regex_valid != 0) {
 		int rc;
 
-		if (au->rules.cnt == 0) {
-			errno = EINVAL;
-			return -1;
-		}
 		r = aup_list_get_cur(&au->le);
 		rc = regexec(&au->regex, r->record, 0, NULL, 0);
 		if (rc == 0)
@@ -932,27 +901,9 @@ static int ausearch_compare(auparse_state_t *au)
 			return 0;
 	}
 	r = aup_list_get_cur(&au->le);
-	if (r) {
-		// for each rule item
-		int results = (au->search_how == AUSEARCH_RULE_AND);
-		aurule_first(&au->rules);
-		do {
-			int tmp;
+	if (r)
+		return expr_eval(au, r, au->expr);
 
-			aurulenode *rule = aurule_get_cur(&au->rules);
-			if (rule == NULL) 
-				continue;
-			tmp = ausearch_interpret_rule(au, r, rule);
-			if (au->search_how == AUSEARCH_RULE_AND) {
-				if (tmp == 0)
-					return 0; /* all must match */
-			} else if (au->search_how <= AUSEARCH_RULE_OR && tmp)
-				return 1; /* Anytime tmp == 1 short circuit */
-		} while (aurule_next(&au->rules));
-		if (results)
-			return 1;
-	}
-	
 	return 0;
 }
 
@@ -961,7 +912,7 @@ int ausearch_next_event(auparse_state_t *au)
 {
 	int rc;
 
-	if (au->rules.cnt == 0) {
+	if (au->expr == NULL && au->regex_valid == 0) {
 		errno = EINVAL;
 		return -1;
 	}
@@ -1015,6 +966,7 @@ int auparse_next_event(auparse_state_t *au)
 			return 0;
 		}
 		if (rc > 0) {	        // Input available
+			rnode *r;
 			if (extract_timestamp(au->cur_buf, &event)) {
 				if (debug)
 					printf("Malformed line:%s\n",
@@ -1056,12 +1008,27 @@ int auparse_next_event(auparse_state_t *au)
 				return 1; // data is available
 			}
 			free((char *)event.host);
+			// Check to see if the event can be emitted due to EOE
+			// or something we know is a single record event. At
+			// this point, new record should be pointed at 'cur'
+			if ((r = aup_list_get_cur(&au->le)) == NULL)
+				continue;
+			if (	r->type == AUDIT_EOE ||
+				r->type < AUDIT_FIRST_EVENT ||
+				r->type >= AUDIT_FIRST_ANOM_MSG) {
+				// Emit the event, set event cursors to 
+				// initial position
+				aup_list_first(&au->le);
+				aup_list_first_field(&au->le);
+				au->parse_state = EVENT_EMITTED;
+				return 1; // data is available
+			}
 		} else {		// Read error
 			return -1;
 		}
 	}	
 }
-
+hidden_def(auparse_next_event)
 
 /* Accessors to event data */
 const au_event_t *auparse_get_timestamp(auparse_state_t *au)
@@ -1163,9 +1130,11 @@ int auparse_first_record(auparse_state_t *au)
 			return rc;
 	}
 	aup_list_first(&au->le);
+	aup_list_first_field(&au->le);
 	
 	return 1;
 }
+hidden_def(auparse_first_record)
 
 
 int auparse_next_record(auparse_state_t *au)
@@ -1180,6 +1149,20 @@ int auparse_next_record(auparse_state_t *au)
 	else
 		return 0;
 }
+hidden_def(auparse_next_record)
+
+
+int auparse_goto_record_num(auparse_state_t *au, unsigned int num)
+{
+	/* Check if a request is out of range */
+	if (num >= aup_list_get_cnt(&au->le))
+		return 0;
+
+	if (aup_list_goto_rec(&au->le, num) != NULL)
+		return 1;
+	else
+		return 0;
+}
 
 
 /* Accessors to record data */
@@ -1267,9 +1250,25 @@ const char *auparse_find_field(auparse_state_t *au, const char *name)
 {
 	free(au->find_field);
 	au->find_field = strdup(name);
-	return auparse_find_field_next(au);
+
+	if (au->le.e.sec) {
+		const char *cur_name;
+		rnode *r;
+
+		// look at current record before moving
+		r = aup_list_get_cur(&au->le);
+		if (r == NULL)
+			return NULL;
+		cur_name = nvlist_get_cur_name(&r->nv);
+		if (cur_name && strcmp(cur_name, name) == 0)
+			return nvlist_get_cur_val(&r->nv);
+
+		return auparse_find_field_next(au);
+	}
+	return NULL;
 }
 
+/* Increment 1 location and then scan for next field */
 const char *auparse_find_field_next(auparse_state_t *au)
 {
 	if (au->find_field == NULL) {
@@ -1277,19 +1276,24 @@ const char *auparse_find_field_next(auparse_state_t *au)
 		return NULL;
 	}
 	if (au->le.e.sec) {
+		int moved = 0;
+
 		rnode *r = aup_list_get_cur(&au->le);
-		while (r) {
-			do {
-				int rc = nvlist_find_name(&r->nv,
-							au->find_field);
-				if (rc) 
-					return nvlist_get_cur_val(&r->nv);
-			} while (nvlist_next(&r->nv));
+		while (r) {	// For each record in the event...
+			if (!moved) {
+				nvlist_next(&r->nv);
+				moved=1;
+			}
+			if (nvlist_find_name(&r->nv, au->find_field))
+				return nvlist_get_cur_val(&r->nv);
 			r = aup_list_next(&au->le);
+			if (r)
+				aup_list_first_field(&au->le);
 		}
 	}
 	return NULL;
 }
+hidden_def(auparse_find_field_next)
 
 
 /* Accessors to field data */
@@ -1313,6 +1317,7 @@ const char *auparse_get_field_str(auparse_state_t *au)
 	}
 	return NULL;
 }
+hidden_def(auparse_get_field_str)
 
 
 int auparse_get_field_int(auparse_state_t *au)
