@@ -1,5 +1,5 @@
 /* auparse.h --
- * Copyright 2006-07 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2006-08 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -32,10 +32,8 @@ extern "C" {
 
 /* Library type definitions */
 
-#ifndef AUPARSE_INTERNAL_HEADER
 /* opaque data type used for maintaining library state */
 typedef struct opaque auparse_state_t;
-#endif
 
 typedef void (*user_destroy)(void *user_data);
 typedef void (*auparse_callback_ptr)(auparse_state_t *au,
@@ -51,6 +49,8 @@ int auparse_reset(auparse_state_t *au);
 void auparse_destroy(auparse_state_t *au);
 
 /* Functions that are part of the search interface */
+int ausearch_add_expression(auparse_state_t *au, const char *expression,
+			    char **error, ausearch_rule_t how);
 int ausearch_add_item(auparse_state_t *au, const char *field, const char *op,
 			const char *value, ausearch_rule_t how);
 int ausearch_add_interpreted_item(auparse_state_t *au, const char *field,
@@ -78,6 +78,7 @@ unsigned int auparse_get_num_records(auparse_state_t *au);
 /* Functions that traverse records in the same event */
 int auparse_first_record(auparse_state_t *au);
 int auparse_next_record(auparse_state_t *au);
+int auparse_goto_record_num(auparse_state_t *au, unsigned int num);
 
 /* Accessors to record data */
 int auparse_get_type(auparse_state_t *au);
