@@ -142,7 +142,6 @@ void databuf_print(DataBuf *db, int print_data, char *fmt, ...)
     }
     printf("\n");
 }
-hidden_def(databuf_print);
 
 int databuf_init(DataBuf *db, size_t size, unsigned flags)
 {
@@ -167,7 +166,6 @@ int databuf_init(DataBuf *db, size_t size, unsigned flags)
 
     return 1;
 }
-hidden_def(databuf_init);
 
 void databuf_free(DataBuf *db)
 {
@@ -185,7 +183,6 @@ void databuf_free(DataBuf *db)
 
     DATABUF_VALIDATE(db);
 }
-hidden_def(databuf_free);
 
 char *databuf_export(DataBuf *db)
 {
@@ -194,7 +191,6 @@ char *databuf_export(DataBuf *db)
     DATABUF_VALIDATE(db);
     return db->alloc_ptr;
 }
-hidden_def(databuf_export);
 
 int databuf_append(DataBuf *db, const char *src, size_t src_size)
 {
@@ -236,7 +232,6 @@ int databuf_append(DataBuf *db, const char *src, size_t src_size)
     DATABUF_VALIDATE(db);
     return 1;
 }
-hidden_def(databuf_append);
 
 int databuf_strcat(DataBuf *db, const char *str)
 {
@@ -264,7 +259,6 @@ int databuf_strcat(DataBuf *db, const char *str)
     DATABUF_VALIDATE(db);
     return 1;
 }
-hidden_def(databuf_strcat);
 
 int databuf_advance(DataBuf *db, size_t advance)
 {
@@ -285,7 +279,6 @@ int databuf_advance(DataBuf *db, size_t advance)
         return -1;
     }
 }
-hidden_def(databuf_advance);
 
 
 int databuf_compress(DataBuf *db)
@@ -305,7 +298,6 @@ int databuf_compress(DataBuf *db)
     DATABUF_VALIDATE(db);
     return 1;
 }
-hidden_def(databuf_compress);
 
 int databuf_reset(DataBuf *db)
 {
@@ -316,7 +308,6 @@ int databuf_reset(DataBuf *db)
     if (debug) databuf_print(db, 1, "databuf_reset() exit");
     return 1;
 }
-hidden_def(databuf_reset);
 
 /*****************************************************************************/
 /*******************************  Test Program  ******************************/
