@@ -1,6 +1,6 @@
 /*
 * ellist.c - Minimal linked list library
-* Copyright (c) 2006-07 Red Hat Inc., Durham, North Carolina.
+* Copyright (c) 2006-08 Red Hat Inc., Durham, North Carolina.
 * All Rights Reserved. 
 *
 * This software may be freely redistributed and/or modified under the
@@ -23,9 +23,13 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <stdio.h>
 #include <errno.h>
 #include <libaudit.h>
 #include "ellist.h"
+#include "interpret.h"
+
+static const char key_sep[2] = { AUDIT_KEY_SEPARATOR, 0 };
 
 void aup_list_create(event_list_t *l)
 {
@@ -37,7 +41,6 @@ void aup_list_create(event_list_t *l)
 	l->e.serial = 0L;
 	l->e.host = NULL;
 }
-hidden_def(aup_list_create);
 
 static void aup_list_last(event_list_t *l)
 {
@@ -58,13 +61,45 @@ rnode *aup_list_next(event_list_t *l)
 		l->cur = l->cur->next;
 	return l->cur;
 }
-hidden_def(aup_list_next);
+
+/*
+ *  * This function does encoding of "untrusted" names just like the kernel
+ *   */
+static char *_audit_c2x(char *final, const char *buf, unsigned int size)
+{
+	unsigned int i;
+	char *ptr = final;
+	const char *hex = "0123456789ABCDEF";
+
+	for (i=0; i<size; i++) {
+		*ptr++ = hex[(buf[i] & 0xF0)>>4]; /* Upper nibble */
+		*ptr++ = hex[buf[i] & 0x0F];      /* Lower nibble */
+	}
+	*ptr = 0;
+	return final;
+}
+
+static char *escape(const char *tmp)
+{
+	char *name;
+	const unsigned char *p = (unsigned char *)tmp;
+	while (*p) {
+		if (*p == '"' || *p < 0x21 || *p > 0x7f) {
+			int len = strlen(tmp);
+			name = malloc((2*len)+1);
+			return _audit_c2x(name, tmp, len);
+		}
+		p++;
+	}
+	asprintf(&name, "\"%s\"", tmp);
+	return name;
+}
 
 /* This funtion does the heavy duty work of splitting a record into
  * its little tiny pieces */
 static int parse_up_record(rnode* r)
 {
-	char *ptr, *buf, *saved;
+	char *ptr, *buf, *saved=NULL;
 	int offset = 0;
 
 	buf = strdup(r->record);
@@ -123,7 +158,24 @@ static int parse_up_record(rnode* r)
 					len--;
 				}
 			}
-			nvlist_append(&r->nv, &n);
+			// Make virtual keys or just store it
+			if (strcmp(n.name, "key") == 0) {
+				char *key = (char *)au_unescape(n.val);
+				char *ptr = strtok_r(key, key_sep, &saved);
+				free(n.name);
+				free(n.val);
+				while (ptr) {
+					n.name = strdup("key");
+					n.val = escape(ptr);
+					nvlist_append(&r->nv, &n);
+					ptr = strtok_r(NULL, key_sep, &saved);
+				}
+				free(key);
+				continue;
+			} else
+				nvlist_append(&r->nv, &n);
+
+			// Do some info gathering for use later
 			if (r->nv.cnt == 1 && strcmp(n.name, "node") == 0)
 				offset = 1; // if node, some positions changes
 			else if (r->nv.cnt == (1 + offset) &&
@@ -204,7 +256,8 @@ static int parse_up_record(rnode* r)
 	return 0;
 }
 
-int aup_list_append(event_list_t *l, char *record, int list_idx, unsigned int line_number)
+int aup_list_append(event_list_t *l, char *record, int list_idx,
+	unsigned int line_number)
 {
 	rnode* r;
 
@@ -243,7 +296,6 @@ int aup_list_append(event_list_t *l, char *record, int list_idx, unsigned int li
 	// Then parse the record up into nvlist
 	return parse_up_record(r);	
 }
-hidden_def(aup_list_append);
 
 void aup_list_clear(event_list_t* l)
 {
@@ -270,7 +322,6 @@ void aup_list_clear(event_list_t* l)
 	free((char *)l->e.host);
 	l->e.host = NULL;      
 }
-hidden_def(aup_list_clear);
 
 /*int aup_list_get_event(event_list_t* l, au_event_t *e)
 {
@@ -285,8 +336,7 @@ hidden_def(aup_list_clear);
 	else
 	        e->host = NULL;
 	return 1;
-}
-hidden_def(aup_list_get_event); */
+} */
 
 int aup_list_set_event(event_list_t* l, au_event_t *e)
 {
@@ -300,7 +350,6 @@ int aup_list_set_event(event_list_t* l, au_event_t *e)
 	e->host = NULL;
 	return 1;
 }
-hidden_def(aup_list_set_event);
 
 rnode *aup_list_find_rec(event_list_t *l, int i)
 {
@@ -316,7 +365,21 @@ rnode *aup_list_find_rec(event_list_t *l, int i)
 	}
 	return NULL;
 }
-hidden_def(aup_list_find_rec);
+
+rnode *aup_list_goto_rec(event_list_t *l, int i)
+{
+        register rnode* window;
+                                                                                
+       	window = l->head;	/* start at the beginning */
+	while (window) {
+		if (window->item == i) {
+			l->cur = window;
+			return window;
+		} else
+			window = window->next;
+	}
+	return NULL;
+}
 
 rnode *aup_list_find_rec_range(event_list_t *l, int low, int high)
 {
@@ -335,7 +398,6 @@ rnode *aup_list_find_rec_range(event_list_t *l, int low, int high)
 	}
 	return NULL;
 }
-hidden_def(aup_list_find_rec_range);
 
 int aup_list_first_field(event_list_t *l)
 {
@@ -345,5 +407,4 @@ int aup_list_first_field(event_list_t *l)
 	} else
 		return 0;
 }
-hidden_def(aup_list_first_field);
 
