@@ -1,5 +1,5 @@
 /* interpret.h --
- * Copyright 2007 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2007,08 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -25,22 +25,28 @@
 
 #include "config.h"
 #include "private.h"
+#include "rnode.h"
 #include <time.h>
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+/* This is a list of field types that we can interpret */
+enum { T_UID, T_GID, T_SYSCALL, T_ARCH, T_EXIT, T_ESCAPED, T_PERM, T_MODE,
+T_SOCKADDR, T_FLAGS, T_PROMISC, T_CAPABILITY, T_SUCCESS, T_A0, T_A1, T_A2,
+T_SIGNAL, T_LIST };
 
 const char *interpret(const rnode *r);
 void aulookup_destroy_uid_list(void);
 void aulookup_destroy_gid_list(void);
+const char *au_unescape(char *buf);
 
 /* Make these hidden to prevent conflicts */
 hidden_proto(interpret);
 hidden_proto(aulookup_destroy_uid_list);
 hidden_proto(aulookup_destroy_gid_list);
-
+hidden_proto(au_unescape);
 
 #ifdef __cplusplus
 }
