@@ -575,6 +575,59 @@ AuParser_reset(AuParser *self)
     return NULL;
 }
 
+/********************************
+ * ausearch_add_expression
+ ********************************/
+PyDoc_STRVAR(search_add_expression_doc,
+"search_add_expression(expression, how) Build up search expression\n\
+\n\
+\n\
+ausearch_add_item adds an expression to the current audit search\n\
+expression.  The search conditions can then be used to scan logs,\n\
+files, or buffers for something of interest.  The expression parameter\n\
+contains an expression, as specified in ausearch-expression(5).\n\
+\n\
+The how parameter determines how this search expression will affect the\n\
+existing search expression, if one is already defined.  The possible\n\
+values are:\n\
+\n\
+AUSEARCH_RULE_CLEAR:\n\
+Clear the current search expression, if any, and use only this search\n\
+expression.\n\
+\n\
+AUSEARCH_RULE_OR:\n\
+\n\
+If a search expression E is already configured, replace it by\n\
+(E || this_search_expression).\n\
+\n\
+AUSEARCH_RULE_AND:\n\
+If a search expression E is already configured, replace it by\n\
+(E && this_search_expression).\n\
+\n\
+No Return value, raises exception (EnvironmentError) on error.\n\
+");
+static PyObject *
+AuParser_search_add_expression(AuParser *self, PyObject *args)
+{
+    const char *expression;
+    char *error;
+    int how;
+    int result;
+
+    if (!PyArg_ParseTuple(args, "si", &expression, &how)) return NULL;
+    PARSER_CHECK;
+
+    result = ausearch_add_expression(self->au, expression, &error, how);
+    if (result == 0) Py_RETURN_NONE;
+    if (error == NULL)
+	PyErr_SetFromErrno(PyExc_EnvironmentError);
+    else {
+	PyErr_SetString(PyExc_EnvironmentError, error);
+	free(error);
+    }
+    return NULL;
+}
+
 /********************************
  * ausearch_add_item
  ********************************/
@@ -582,8 +635,8 @@ PyDoc_STRVAR(search_add_item_doc,
 "search_add_item(field, op, value, how) Build up search rule\n\
 \n\
 \n\
-search_add_item() adds one search condition to the audit search\n\
-API. The search conditions can then be used to scan logs, files, or\n\
+search_add_item() adds one search condition to the current audit search\n\
+expression. The search conditions can then be used to scan logs, files, or\n\
 buffers for something of interest. The field value is the field name\n\
 that the value will be checked for. The op variable describes what\n\
 kind of check is to be done. Legal op values are:\n\
@@ -601,23 +654,22 @@ it is NOT equal to the value given in this rule.\n\
 \n\
 The value parameter is compared to the uninterpreted field value.\n\
 \n\
-The how parameter determines how this search condition will be stored\n\
-internally. The possible values are:\n\
+The how parameter determines how this search expression will affect the\n\
+existing search expression, if one is already defined.  The possible\n\
+values are:\n\
 \n\
 AUSEARCH_RULE_CLEAR:\n\
-When this is used, it clears any previous search condition and inserts\n\
-it as the first one.\n\
+Clear the current search expression, if any, and use only this search\n\
+expression.\n\
 \n\
 AUSEARCH_RULE_OR:\n\
 \n\
-When this is used, it means that the results of its evaluation will be\n\
-or'ed with other search conditions.\n\
+If a search expression E is already configured, replace it by\n\
+(E || this_search_expression).\n\
 \n\
 AUSEARCH_RULE_AND:\n\
-When this is used, it means that the results of its evaluation will be\n\
-and'ed with other search conditions.\n\
-\n\
-All search conditions must be the same type, you cannot mix 'and' and 'or'.\n\
+If a search expression E is already configured, replace it by\n\
+(E && this_search_expression).\n\
 \n\
 No Return value, raises exception (EnvironmentError) on error.\n\
 ");
@@ -647,10 +699,10 @@ PyDoc_STRVAR(search_add_interpreted_item_doc,
 "search_add_interpreted_item(field, op, value, how) Build up search rule\n\
 \n\
 \n\
-search_add_interpreted_item() adds one search condition to the audit search\n\
-API. The search conditions can then be used to scan logs, files, or\n\
-buffers for something of interest. The field value is the field name\n\
-that the value will be checked for. The op variable describes what\n\
+search_add_interpreted_item() adds one search condition to the current audit\n\
+search expression. The search conditions can then be used to scan logs,\n\
+files, or buffers for something of interest. The field value is the field\n\
+name that the value will be checked for. The op variable describes what\n\
 kind of check is to be done. Legal op values are:\n\
 \n\
 'exists':\n\
@@ -667,23 +719,22 @@ it is NOT equal to the value given in this rule.\n\
 The value parameter is compared to the interpreted field value (the value\n\
 that would be returned by AuParser.interpret_field).\n\
 \n\
-The how parameter determines how this search condition will be stored\n\
-internally. The possible values are:\n\
+The how parameter determines how this search expression will affect the\n\
+existing search expression, if one is already defined.  The possible\n\
+values are:\n\
 \n\
 AUSEARCH_RULE_CLEAR:\n\
-When this is used, it clears any previous search condition and inserts\n\
-it as the first one.\n\
+Clear the current search expression, if any, and use only this search\n\
+expression.\n\
 \n\
 AUSEARCH_RULE_OR:\n\
 \n\
-When this is used, it means that the results of its evaluation will be\n\
-or'ed with other search conditions.\n\
+If a search expression E is already configured, replace it by\n\
+(E || this_search_expression).\n\
 \n\
 AUSEARCH_RULE_AND:\n\
-When this is used, it means that the results of its evaluation will be\n\
-and'ed with other search conditions.\n\
-\n\
-All search conditions must be the same type, you cannot mix 'and' and 'or'.\n\
+If a search expression E is already configured, replace it by\n\
+(E && this_search_expression).\n\
 \n\
 No Return value, raises exception (EnvironmentError) on error.\n\
 ");
@@ -713,31 +764,30 @@ PyDoc_STRVAR(search_add_timestamp_item_doc,
 "search_add_timestamp_item(op, sec, milli, how) Build up search rule\n\
 \n\
 \n\
-search_add_timestamp_item adds an event time condition to the audit\n\
-search state. The search conditions can then be used to scan logs, files,\n\
-or buffers for something of interest. The op parameter specifies the\n\
+search_add_timestamp_item adds an event time condition to the current audit\n\
+search expression. The search conditions can then be used to scan logs,\n\
+files, or buffers for something of interest. The op parameter specifies the\n\
 desired comparison. Legal op values are \"<\", \"<=\", \">=\", \">\" and\n\
 \"=\". The left operand of the comparison operator is the timestamp of the\n\
 examined event, the right operand is specified by the sec and milli\n\
 parameters.\n\
 \n\
-The how parameter determines how this search condition will be stored\n\
-internally. The possible values are:\n\
+The how parameter determines how this search expression will affect the\n\
+existing search expression, if one is already defined.  The possible\n\
+values are:\n\
 \n\
 AUSEARCH_RULE_CLEAR:\n\
-When this is used, it clears any previous search condition and inserts\n\
-it as the first one.\n\
+Clear the current search expression, if any, and use only this search\n\
+expression.\n\
 \n\
 AUSEARCH_RULE_OR:\n\
 \n\
-When this is used, it means that the results of its evaluation will be\n\
-or'ed with other search conditions.\n\
+If a search expression E is already configured, replace it by\n\
+(E || this_search_expression).\n\
 \n\
 AUSEARCH_RULE_AND:\n\
-When this is used, it means that the results of its evaluation will be\n\
-and'ed with other search conditions.\n\
-\n\
-All search conditions must be the same type, you cannot mix 'and' and 'or'.\n\
+If a search expression E is already configured, replace it by\n\
+(E && this_search_expression).\n\
 \n\
 No Return value, raises exception (EnvironmentError) on error.\n\
 ");
@@ -1006,6 +1056,37 @@ AuParser_next_record(AuParser *self)
     return NULL;
 }
 
+/********************************
+ * auparse_goto_record_num
+ ********************************/
+PyDoc_STRVAR(goto_record_num_doc,
+"goto_record_num() Move record cursor to specific position.\n\
+\n\
+goto_record_num() will move the internal library cursors to point\n\
+to a specific physical record number. Records within the same event are\n\
+numbered  starting  from  0. This is generally not needed but there are\n\
+some cases where one may want precise control  over  the  exact  record\n\
+being looked at.\n\
+\n\
+Returns True on success, False if no more records in current event\n\
+Raises exception (EnvironmentError) on error.\n\
+");
+static PyObject *
+AuParser_goto_record_num(AuParser *self, PyObject *args)
+{
+    int result;
+    unsigned int num;
+
+    if (!PyArg_ParseTuple(args, "i", &num)) return NULL;
+    PARSER_CHECK;
+    result = auparse_goto_record_num(self->au, num);
+
+    if (result >  0) Py_RETURN_TRUE;
+    if (result == 0) Py_RETURN_FALSE;
+    PyErr_SetFromErrno(PyExc_EnvironmentError);
+    return NULL;
+}
+
 /********************************
  * auparse_get_type
  ********************************/
@@ -1371,6 +1452,7 @@ static PyMethodDef AuParser_methods[] = {
     {"flush_feed",        (PyCFunction)AuParser_flush_feed,        METH_NOARGS,  flush_feed_doc},
     {"add_callback",      (PyCFunction)AuParser_add_callback,      METH_VARARGS, add_callback_doc},
     {"reset",             (PyCFunction)AuParser_reset,             METH_NOARGS,  reset_doc},
+    {"search_add_expression", (PyCFunction)AuParser_search_add_expression, METH_VARARGS, search_add_expression_doc},
     {"search_add_item",   (PyCFunction)AuParser_search_add_item,   METH_VARARGS, search_add_item_doc},
     {"search_add_interpreted_item", (PyCFunction)AuParser_search_add_interpreted_item, METH_VARARGS, search_add_interpreted_item_doc},
     {"search_add_timestamp_item", (PyCFunction)AuParser_search_add_timestamp_item, METH_VARARGS, search_add_timestamp_item_doc},
@@ -1383,6 +1465,7 @@ static PyMethodDef AuParser_methods[] = {
     {"get_num_records",   (PyCFunction)AuParser_get_num_records,   METH_NOARGS,  get_num_records_doc},
     {"first_record",      (PyCFunction)AuParser_first_record,      METH_NOARGS,  first_record_doc},
     {"next_record",       (PyCFunction)AuParser_next_record,       METH_NOARGS,  next_record_doc},
+    {"goto_record_num",   (PyCFunction)AuParser_goto_record_num,   METH_VARARGS,  goto_record_num_doc},
     {"get_type",          (PyCFunction)AuParser_get_type,          METH_NOARGS,  get_type_doc},
     {"get_line_number",   (PyCFunction)AuParser_get_line_number,   METH_NOARGS,  get_line_number_doc},
     {"get_filename",      (PyCFunction)AuParser_get_filename,      METH_NOARGS,  get_filename_doc},
