@@ -1,5 +1,5 @@
 /* audit_logging.c -- 
- * Copyright 2005-2007 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2005-2008 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -31,6 +31,7 @@
 #include <netdb.h>	// gethostbyname
 #include <arpa/inet.h>	// inet_ntop
 #include <utmp.h>
+#include <limits.h>	// PATH_MAX
 
 #include "libaudit.h"
 #include "private.h"
@@ -378,10 +379,10 @@ int audit_log_acct_message(int audit_fd, int type, const char *pgname,
 		}
 		if (enc)
 			format = 
-	 "op=%s acct=\"%s\" exe=%s (hostname=%s, addr=%s, terminal=%s res=%s)";
+	     "op=%s acct=%s exe=%s (hostname=%s, addr=%s, terminal=%s res=%s)";
 		else
 			format = 
-	     "op=%s acct=%s exe=%s (hostname=%s, addr=%s, terminal=%s res=%s)";
+	 "op=%s acct=\"%s\" exe=%s (hostname=%s, addr=%s, terminal=%s res=%s)";
 
 		snprintf(buf, sizeof(buf), format,
 			op, user, pgname,
@@ -561,7 +562,7 @@ int audit_log_semanage_message(int audit_fd, int type, const char *pgname,
 			);
 	} else
 		snprintf(buf, sizeof(buf),
-		"op=%s id=%u old seuser=%s old role=%s old range=%s new seuser=%s new role=%s new range=%s exe=%s (hostname=%s, addr=%s, terminal=%s res=%s)",
+		"op=%s id=%u old-seuser=%s old-role=%s old-range=%s new-seuser=%s new-role=%s new-range=%s exe=%s (hostname=%s, addr=%s, terminal=%s res=%s)",
 			op, id,
 			old_seuser && strlen(old_seuser) ? old_seuser : "?",
 			old_role && strlen(old_role) ? old_role : "?",
@@ -631,8 +632,7 @@ int audit_log_user_command(int audit_fd, int type, const char *command,
 
 	// We borrow the commname buffer
 	if (getcwd(commname, PATH_MAX) == NULL)
-		strcpy(cwdname, "?");
-	strcpy(cwdname, commname);
+		strcpy(commname, "?");
 	p = commname;
 	len = strlen(commname);
 	while (*p) {
@@ -643,6 +643,8 @@ int audit_log_user_command(int audit_fd, int type, const char *command,
 		}
 		p++;
 	}
+	if (cwdenc == 0)
+		strcpy(cwdname, commname);
 
 	len = strlen(cmd);
 	// Trim the trailing carriage return and spaces
@@ -652,7 +654,10 @@ int audit_log_user_command(int audit_fd, int type, const char *command,
 	}
 
 	p = cmd;
-	strcpy(commname, cmd);
+	if (len >= PATH_MAX) {
+		cmd[PATH_MAX] = 0;
+		len = PATH_MAX-1;
+	}
 	while (*p) {
 		if (*p == '"' || *p < 0x21 || (unsigned)*p > 0x7f) {
 			_audit_c2x(commname, cmd, len);
@@ -661,7 +666,11 @@ int audit_log_user_command(int audit_fd, int type, const char *command,
 		}
 		p++;
 	}
+	if (cmdenc == 0)
+		strcpy(commname, cmd);
+	free(cmd);
 
+	// Make the format string
 	if (cwdenc)
 		p=stpcpy(format, "cwd=%s ");
 	else
@@ -674,13 +683,13 @@ int audit_log_user_command(int audit_fd, int type, const char *command,
 
 	strcpy(p, "(terminal=%s res=%s)");
 
+	// now use the format string to make the event
 	snprintf(buf, sizeof(buf), format,
 			cwdname, commname,
 			tty ? tty : "?",
 			success
 		);
 
-	free(cmd);
 	errno = 0;
 	ret = audit_send_user_message( audit_fd, type, HIDE_IT, buf );
 	if ((ret < 1) && errno == 0)
