@@ -1,5 +1,5 @@
 /* i386_table.h --
- * Copyright 2005-07 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2005-08 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -343,4 +343,6 @@ _S(321, "signalfd")
 _S(322, "timerfd")
 _S(323, "eventfd")
 _S(324, "fallocate")
+_S(325, "timerfd_settime")
+_S(326, "timerfd_gettime")
 
