@@ -1,5 +1,5 @@
 /* ia64_table.h --
- * Copyright 2005-07 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2005-08 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -300,4 +300,7 @@ _S(1306, "utimensat")
 _S(1307, "signalfd")
 _S(1308, "timerfd")
 _S(1309, "eventfd")
+_S(1310, "timerfd_create")
+_S(1311, "timerfd_settime")
+_S(1312, "timerfd_gettime")
 
