@@ -35,6 +35,7 @@
 #include <sys/poll.h>
 #include <sys/utsname.h>
 #include <fcntl.h>	/* O_NOFOLLOW needs gnu defined */
+#include <limits.h>	/* for PATH_MAX */
 
 #include "libaudit.h"
 #include "private.h"
@@ -74,9 +75,9 @@ static const struct nv_list failure_actions[] =
   { NULL,		0 }
 };
 
-int audit_archadded = 0;
-int audit_syscalladded = 0;
-unsigned int audit_elf = 0U;
+int audit_archadded hidden = 0;
+int audit_syscalladded hidden = 0;
+unsigned int audit_elf hidden = 0U;
 static struct libaudit_conf config;
 
 static int audit_failure_parser(const char *val, int line);
@@ -90,7 +91,7 @@ static const struct kw_pair keywords[] =
 };
 
 /* FIXME: Make this static again after deprecated functions no longer need it */
-int audit_priority(int xerrno)
+int hidden audit_priority(int xerrno)
 {
 	/* If they've compiled their own kernel and did not include
 	 * the audit susbsystem, they will get ECONNREFUSED. We'll
@@ -109,6 +110,7 @@ int audit_request_status(int fd)
 			"Error sending status request (%s)", strerror(-rc));
 	return rc;
 }
+hidden_def(audit_request_status)
 
 /*
  * Set everything to its default value
@@ -580,6 +582,7 @@ int audit_add_watch_dir(int type, struct audit_rule_data **rulep,
 
 	return  0;
 }
+hidden_def(audit_add_watch_dir)
 
 int audit_add_rule_data(int fd, struct audit_rule_data *rule,
                         int flags, int action)
@@ -735,6 +738,7 @@ int audit_rule_syscall_data(struct audit_rule_data *rule, int scall)
 	rule->mask[word] |= bit;
 	return 0;
 }
+hidden_def(audit_rule_syscall_data)
 
 int audit_rule_syscallbyname_data(struct audit_rule_data *rule,
                                   const char *scall)
@@ -762,6 +766,7 @@ int audit_rule_syscallbyname_data(struct audit_rule_data *rule,
 		return audit_rule_syscall_data(rule, nr);
 	return -1;
 }
+hidden_def(audit_rule_syscallbyname_data)
 
 int audit_rule_fieldpair_data(struct audit_rule_data **rulep, const char *pair,
                               int flags)
@@ -815,7 +820,6 @@ int audit_rule_fieldpair_data(struct audit_rule_data **rulep, const char *pair,
 	if (v == NULL || f == v)
 		return -1;
 
-	audit_msg(LOG_DEBUG,"pair=%s\n", f);
 	if ((field = audit_name_to_field(f)) < 0) 
 		return -2;
 
@@ -823,7 +827,6 @@ int audit_rule_fieldpair_data(struct audit_rule_data **rulep, const char *pair,
 	if (flags == AUDIT_FILTER_EXCLUDE && field != AUDIT_MSGTYPE)
 		return -12; 
 
-	audit_msg(LOG_DEBUG,"f%d%s%s\n", field, audit_operator_to_symbol(op),v);
 	rule->fields[rule->field_count] = field;
 	rule->fieldflags[rule->field_count] = op;
 	switch (field)
@@ -1057,6 +1060,13 @@ int audit_rule_fieldpair_data(struct audit_rule_data **rulep, const char *pair,
 				rule->values[rule->field_count] = val;
 			}
 			break;
+		case AUDIT_FILETYPE:
+			rule->values[rule->field_count] = 
+				audit_name_to_ftype(v);
+			if (rule->values[rule->field_count] < 0) {
+				return -16;
+			}
+			break;
 		case AUDIT_DEVMAJOR...AUDIT_INODE:
 		case AUDIT_SUCCESS...AUDIT_SUCCESS:
 			if (flags == AUDIT_FILTER_ENTRY)
@@ -1112,4 +1122,4 @@ int audit_detect_machine(void)
 		return audit_name_to_machine(uts.machine);
 	return -1;
 }
-
+hidden_def(audit_detect_machine)
