@@ -83,6 +83,7 @@ extern "C" {
 #define AUDIT_USER_CMD		1123	/* User shell command and args */
 #define AUDIT_USER_TTY		1124	/* Non-ICANON TTY input meaning */
 #define AUDIT_CHUSER_ID		1125	/* Changed user ID supplemental data */
+#define AUDIT_GRP_AUTH		1126	/* Authentication for group password */
 
 #define AUDIT_FIRST_DAEMON	1200
 #define AUDIT_LAST_DAEMON	1299
@@ -210,6 +211,7 @@ extern "C" {
 #define AUDIT_DEV_ALLOC			2307 /* Device was allocated */
 #define AUDIT_DEV_DEALLOC		2308 /* Device was deallocated */
 #define AUDIT_FS_RELABEL		2309 /* Filesystem relabeled */
+#define AUDIT_USER_MAC_POLICY_LOAD	2310 /* Userspc daemon loaded policy */
 
 #define AUDIT_FIRST_CRYPTO_MSG		2400
 #define AUDIT_LAST_CRYPTO_MSG		2499
@@ -278,11 +280,6 @@ extern "C" {
 #define AUDIT_WATCH         105	/* This is a field in syscall rule */
 #endif
 
-/* This is from the directory auditing patch */
-#ifndef AUDIT_DIR
-#define AUDIT_DIR       107
-#endif
-
 /* Defines for syscall classes - watch permissions */
 #ifndef AUDIT_PERM
 #define AUDIT_PERM		106	/* This is a field in syscall rule */
@@ -292,11 +289,22 @@ extern "C" {
 #define AUDIT_PERM_ATTR		8
 #endif
 
+/* This is from the directory auditing patch */
+#ifndef AUDIT_DIR
+#define AUDIT_DIR           107
+#endif
+
+/* This is from the filetype patch */
+#ifndef AUDIT_FILETYPE
+#define AUDIT_FILETYPE      108
+#endif
+
 /* This is from filterkey patch */
 #ifndef AUDIT_FILTERKEY
 #define AUDIT_FILTERKEY     210
 #define AUDIT_MAX_KEY_LEN   32
 #endif
+#define AUDIT_KEY_SEPARATOR 0x01
 
 /* This is new list defines from audit.h */
 #ifndef AUDIT_FILTER_USER
@@ -466,6 +474,8 @@ extern int          audit_elf_to_machine(unsigned int elf);
 extern const char *audit_operator_to_symbol(int op);
 extern int        audit_name_to_errno(const char *error);
 extern const char *audit_errno_to_name(int error);
+extern int        audit_name_to_ftype(const char *name);
+extern const char *audit_ftype_to_name(int ftype); 
 
 /* AUDIT_GET */
 extern int audit_request_status(int fd);
