@@ -67,6 +67,7 @@ _S(AUDIT_USER_SELINUX_ERR,           "USER_SELINUX_ERR"              )
 _S(AUDIT_USER_CMD,                   "USER_CMD"                      )
 _S(AUDIT_USER_TTY,                   "USER_TTY"                      )
 _S(AUDIT_CHUSER_ID,                  "CHUSER_ID"                     )
+_S(AUDIT_GRP_AUTH,                   "GRP_AUTH"                      )
 _S(AUDIT_DAEMON_START,               "DAEMON_START"                  )
 _S(AUDIT_DAEMON_END,                 "DAEMON_END"                    )
 _S(AUDIT_DAEMON_ABORT,               "DAEMON_ABORT"                  )
@@ -161,4 +162,5 @@ _S(AUDIT_USER_UNLABELED_EXPORT,      "USER_UNLABELED_EXPORT"         )
 _S(AUDIT_DEV_ALLOC,                  "DEV_ALLOC"                     )
 _S(AUDIT_DEV_DEALLOC,                "DEV_DEALLOC"                   )
 _S(AUDIT_FS_RELABEL,                 "FS_RELABEL"                    )
+_S(AUDIT_USER_MAC_POLICY_LOAD,       "USER_MAC_POLICY_LOAD"          )
 
