@@ -130,6 +130,7 @@ retry:
 		len = -errno;
 	return len; 
 }
+hidden_def(audit_get_reply)
 
 
 /* 
@@ -245,7 +246,6 @@ int audit_send(int fd, int type, const void *data, unsigned int size)
 
 	return 0;
 }
-hidden_def(audit_send)
 
 /*
  * This function will take a peek into the next packet and see if there's
