@@ -1,5 +1,5 @@
 /* ppc_table.h --
- * Copyright 2005-07 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2005-08 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -323,4 +323,7 @@ _S(306, "timerfd")
 _S(307, "eventfd")
 _S(308, "sync_file_range2")
 _S(309, "fallocate")
+_S(310, "subpage_prot")
+_S(311, "timerfd_settime")
+_S(312, "timerfd_gettime")
 
