@@ -1,5 +1,5 @@
 /* s390x_table.h --
- * Copyright 2005,2006 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2005,2006,2008 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -226,6 +226,12 @@ _S(259, "clock_settime")
 _S(260, "clock_gettime")
 _S(261, "clock_getres")
 _S(262, "clock_nanosleep")
+_S(265, "statfs64")
+_S(266, "fstatfs64")
+_S(267, "remap_file_pages")
+//_S(268, "")
+//_S(269, "")
+//_S(270, "")
 _S(271, "mq_open")
 _S(272, "mq_unlink")
 _S(273, "mq_timedsend")
@@ -274,4 +280,7 @@ _S(315, "utimensat")
 _S(316, "signalfd")
 _S(317, "timerfd")
 _S(318, "eventfd")
+_S(319, "timerfd_create")
+_S(320, "timerfd_settime")
+_S(321, "timerfd_gettime")
 
