@@ -1,5 +1,5 @@
 /* aureport-options.c - parse commandline options and configure aureport
- * Copyright 2005-07 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2005-08 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -36,6 +36,7 @@
 
 /* Global vars that will be accessed by the main program */
 char *user_file = NULL;
+int force_logs = 0;
 
 /* These are for compatibility with parser */
 unsigned int event_id = -1;
@@ -66,7 +67,8 @@ enum {  R_INFILE, R_TIME_END, R_TIME_START, R_VERSION, R_SUMMARY, R_LOG_TIMES,
 	R_CONFIGS, R_LOGINS, R_USERS, R_TERMINALS, R_HOSTS, R_EXES, R_FILES,
 	R_AVCS, R_SYSCALLS, R_PIDS, R_EVENTS, R_ACCT_MODS,  
 	R_INTERPRET, R_HELP, R_ANOMALY, R_RESPONSE, R_SUMMARY_DET, R_CRYPTO,
-	R_MAC, R_FAILED, R_SUCCESS, R_ADD, R_DEL, R_AUTH, R_NODE };
+	R_MAC, R_FAILED, R_SUCCESS, R_ADD, R_DEL, R_AUTH, R_NODE, R_IN_LOGS,
+	R_KEYS };
 
 static struct nv_pair optiontab[] = {
 	{ R_AUTH, "-au" },
@@ -91,6 +93,9 @@ static struct nv_pair optiontab[] = {
 	{ R_INTERPRET, "--interpret" },
 	{ R_INFILE, "-if" },
 	{ R_INFILE, "--input" },
+	{ R_IN_LOGS, "--input-logs" },
+	{ R_KEYS, "-k" },
+	{ R_KEYS, "--key" },
 	{ R_LOGINS, "-l" },
 	{ R_LOGINS, "--login" },
 	{ R_ACCT_MODS, "-m" },
@@ -150,7 +155,9 @@ static void usage(void)
 	"\t--help\t\t\t\thelp\n"
 	"\t-i,--interpret\t\t\tInterpretive mode\n"
 	"\t-if,--input <Input File name>\tuse this file as input\n"
+	"\t--input-logs\t\t\tUse the logs even if stdin is a pipe\n"
 	"\t-l,--login\t\t\tLogin report\n"
+	"\t-k,--key\t\t\tKey report\n"
 	"\t-m,--mods\t\t\tModification to accounts report\n"
 	"\t-ma,--mac\t\t\tMandatory Access Control (MAC) report\n"
 	"\t--node <node name>\t\tOnly events from a specific node\n"
@@ -447,6 +454,19 @@ int check_params(int count, char *vars[])
 				}
 			}
 			break;
+		case R_KEYS:
+			if (set_report(RPT_KEY))
+				retval = -1;
+			else {
+				if (!optarg) {
+					set_detail(D_DETAILED);
+					event_exe = dummy;
+					event_key = dummy;
+				} else {
+					UNIMPLEMENTED;
+				}
+			}
+			break;
 		case R_TIME_END:
 			if (optarg) {
 				if ( (c+2 < count) && vars[c+2] && 
@@ -559,6 +579,9 @@ int check_params(int count, char *vars[])
 		case R_DEL:
 			event_conf_act = C_DEL;
 			break;
+		case R_IN_LOGS:
+			force_logs = 1;
+			break;
 		case R_VERSION:
 	                printf("aureport version %s\n", VERSION);
 			exit(0);
@@ -586,6 +609,7 @@ int check_params(int count, char *vars[])
 				event_hostname = dummy;
 				event_terminal = dummy;
 				event_exe = dummy;
+				event_key = dummy;
 			}
 		}
 	} else
