@@ -1,6 +1,6 @@
 /*
  * aureport.c - main file for aureport utility 
- * Copyright 2005-07 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2005-08 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -54,6 +54,7 @@ static int str2event(char *s, event *e);
 static int events_are_equal(event *e1, event *e2);
 
 extern char *user_file;
+extern int force_logs;
 
 static int input_is_pipe(void)
 {
@@ -107,10 +108,12 @@ int main(int argc, char *argv[])
 	}
 		
 	print_title();
-	if (input_is_pipe())
-		rc = process_stdin();
-	else if (user_file)
+	if (user_file)
 		rc = process_file(user_file);
+	else if (force_logs)
+		rc = process_logs(&config);
+	else if (input_is_pipe())
+		rc = process_stdin();
 	else
 		rc = process_logs(&config);
 	if (rc) {
