@@ -1,5 +1,5 @@
 /* ausearch-options.c - parse commandline options and configure ausearch
- * Copyright 2005-07 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2005-08 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -36,6 +36,7 @@
 
 /* Global vars that will be accessed by the main program */
 char *user_file = NULL;
+int force_logs = 0;
 
 /* Global vars that will be accessed by the match model */
 unsigned int event_id = -1;
@@ -47,6 +48,8 @@ int event_exact_match = 0;
 uid_t event_uid = -1, event_euid = -1, event_loginuid = -1;
 int event_syscall = -1;
 int event_ua = 0, event_ga = 0, event_se = 0;
+int just_one = 0;
+int event_session_id = -1;
 const char *event_key = NULL;
 const char *event_node = NULL;
 const char *event_filename = NULL;
@@ -68,7 +71,7 @@ enum { S_EVENT, S_COMM, S_FILENAME, S_ALL_GID, S_EFF_GID, S_GID, S_HELP,
 S_HOSTNAME, S_INTERP, S_INFILE, S_MESSAGE_TYPE, S_PID, S_SYSCALL, S_OSUCCESS,
 S_TIME_END, S_TIME_START, S_TERMINAL, S_ALL_UID, S_EFF_UID, S_UID, S_LOGINID,
 S_VERSION, S_EXACT_MATCH, S_EXECUTABLE, S_CONTEXT, S_SUBJECT, S_OBJECT,
-S_PPID, S_KEY, S_RAW, S_NODE };
+S_PPID, S_KEY, S_RAW, S_NODE, S_IN_LOGS, S_JUST_ONE, S_SESSION };
 
 static struct nv_pair optiontab[] = {
 	{ S_EVENT, "-a" },
@@ -91,6 +94,8 @@ static struct nv_pair optiontab[] = {
 	{ S_INTERP, "--interpret" },
 	{ S_INFILE, "-if" },
 	{ S_INFILE, "--input" },
+	{ S_IN_LOGS, "--input-logs" },
+	{ S_JUST_ONE, "--just-one" },
 	{ S_KEY, "-k" },
 	{ S_KEY, "--key" },
 	{ S_MESSAGE_TYPE, "-m" },
@@ -109,6 +114,7 @@ static struct nv_pair optiontab[] = {
 	{ S_SYSCALL, "--syscall" },
 	{ S_CONTEXT, "-se" },
 	{ S_CONTEXT, "--context" },
+	{ S_SESSION, "--session" },
 	{ S_SUBJECT, "-su" },
 	{ S_SUBJECT, "--subject" },
 	{ S_OSUCCESS, "-sv" },
@@ -160,6 +166,8 @@ static void usage(void)
 	"\t-hn,--host <Host Name>\t\tsearch based on remote host name\n"
 	"\t-i,--interpret\t\t\tInterpret results to be human readable\n"
 	"\t-if,--input <Input File name>\tuse this file instead of current logs\n"
+	"\t--input-logs\t\t\tUse the logs even if stdin is a pipe\n"
+	"\t--just-one\t\t\tEmit just one event\n"
 	"\t-k,--key  <key string>\t\tsearch based on key field\n"
 	"\t-m,--message  <Message type>\tsearch based on message type\n"
 	"\t-n,--node  <Node name>\t\tsearch based on machine's name\n"
@@ -169,6 +177,7 @@ static void usage(void)
 	"\t-r,--raw\t\t\toutput is completely unformatted\n"
 	"\t-sc,--syscall <SysCall name>\tsearch based on syscall name or number\n"
 	"\t-se,--context <SE Linux context> search based on either subject or\n\t\t\t\t\t object\n"
+	"\t--session <login session id>\tsearch based on login session id\n"
 	"\t-su,--subject <SE Linux context> search based on context of the Subject\n"
 	"\t-sv,--success <Success Value>\tsearch based on syscall or event\n\t\t\t\t\tsuccess value\n"
 	"\t-te,--end [end date] [end time]\tending date & time for search\n"
@@ -669,6 +678,27 @@ int check_params(int count, char *vars[])
                 	}
 			c++;
 			break;
+		case S_SESSION:
+			if (!optarg) {
+				fprintf(stderr, 
+					"Argument is required for %s\n",
+					vars[c]);
+				retval = -1;
+				break;
+			}
+			if (isdigit(optarg[0])) {
+				errno = 0;
+				event_session_id = strtol(optarg,NULL,10);
+				if (errno)
+					retval = -1;
+				c++;
+			} else {
+				fprintf(stderr, 
+				"Session id must be a numeric value, was %s\n",
+					optarg);
+				retval = -1;
+			}
+			break;
 		case S_TIME_END:
 			if (optarg) {
 				if ( (c+2 < count) && vars[c+2] && 
@@ -905,6 +935,12 @@ int check_params(int count, char *vars[])
 		case S_EXACT_MATCH:
 			event_exact_match=1;
 			break;
+		case S_IN_LOGS:
+			force_logs = 1;
+			break;
+		case S_JUST_ONE:
+			just_one = 1;
+			break;
 		case S_EXECUTABLE:
 			if (!optarg) {
 				fprintf(stderr, 
