@@ -1,6 +1,6 @@
 /*
 * ausearch-parse.c - Extract interesting fields and check for match
-* Copyright (c) 2005-07 Red Hat Inc., Durham, North Carolina.
+* Copyright (c) 2005-08 Red Hat Inc., Durham, North Carolina.
 * All Rights Reserved. 
 *
 * This software may be freely redistributed and/or modified under the
@@ -30,11 +30,13 @@
 #include <sys/socket.h>
 #include <sys/un.h>
 #include <netdb.h>
+#include <limits.h>	/* PATH_MAX */
 #include "libaudit.h"
 #include "ausearch-options.h"
 #include "ausearch-parse.h"
 
 #define NAME_OFFSET 36
+static const char key_sep[2] = { AUDIT_KEY_SEPARATOR, 0 };
 
 static int parse_syscall(lnode *n, search_items *s);
 static int parse_dir(const lnode *n, search_items *s);
@@ -168,7 +170,6 @@ int extract_search_items(llist *l)
 				// If we don't know, don't extract.
 				return 0;
 			}
-
 			switch (n->type) {
 			case AUDIT_SYSCALL:
 				ret = parse_syscall(n, s);
@@ -407,6 +408,20 @@ static int parse_syscall(lnode *n, search_items *s)
 			*term = ' ';
 		}
 	}
+	// ses
+	str = strstr(term, "ses=");
+	if (str) {
+		ptr = str + 4;
+		term = strchr(ptr, ' ');
+		if (term == NULL)
+			return 32;
+		*term = 0;
+		errno = 0;
+		s->session_id = strtoul(ptr, NULL, 10);
+		if (errno)
+			return 33;
+		*term = ' ';
+	}
 	if (event_comm) {
 		// dont do this search unless needed
 		str = strstr(term, "comm=");
@@ -419,14 +434,14 @@ static int parse_syscall(lnode *n, search_items *s)
 				str++;
 				term = strchr(str, '"');
 				if (term == NULL)
-					return 32;
+					return 34;
 				*term = 0;
 				s->comm = strdup(str);
 				*term = '"';
 			} else 
 				s->comm = unescape(str);
 		} else
-			return 33;
+			return 35;
 	}
 	if (event_exe) {
 		// dont do this search unless needed
@@ -437,14 +452,14 @@ static int parse_syscall(lnode *n, search_items *s)
 				str++;
 				term = strchr(str, '"');
 				if (term == NULL)
-					return 34;
+					return 36;
 				*term = 0;
 				s->exe = strdup(str);
 				*term = '"';
 			} else 
 				s->exe = unescape(str);
 		} else
-			return 35;
+			return 37;
 	}
 	if (event_subject) {
 		// scontext
@@ -453,7 +468,7 @@ static int parse_syscall(lnode *n, search_items *s)
 			str += 5;
 			term = strchr(str, ' ');
 			if (term == NULL)
-				return 36;
+				return 38;
 			*term = 0;
 			if (audit_avc_init(s) == 0) {
 				anode an;
@@ -463,23 +478,55 @@ static int parse_syscall(lnode *n, search_items *s)
 				alist_append(s->avc, &an);
 				*term = ' ';
 			} else
-				return 37;
+				return 39;
 		}
 	}
 	if (event_key) {
 		str = strstr(term, "key=");
 		if (str) {
+			if (!s->key) {
+				//create
+				s->key = malloc(sizeof(slist));
+				if (s->key == NULL)
+					return 40;
+				slist_create(s->key);
+			}
 			str += 4;
 			if (*str == '"') {
 				str++;
 				term = strchr(str, '"');
 				if (term == NULL)
-					return 38;
+					return 41;
 				*term = 0;
-				s->key = strdup(str);
+				if (s->key) {
+					// append
+					snode sn;
+					sn.str = strdup(str);
+					sn.key = NULL;
+					sn.hits = 1;
+					slist_append(s->key, &sn);
+				}
 				*term = '"';
-			} else 
-				s->key = unescape(str);
+			} else { 
+				if (s->key) {
+					char *saved=NULL;
+					char *keyptr = unescape(str);
+					char *kptr = strtok_r(keyptr,
+							key_sep, &saved);
+					while (kptr) {
+						snode sn;
+						// append
+						sn.str = strdup(kptr);
+						sn.key = NULL;
+						sn.hits = 1;
+						slist_append(s->key, &sn);
+						kptr = strtok_r(NULL,
+							key_sep, &saved);
+					}
+					free(keyptr);
+
+				}
+			}
 		}
 	}
 	return 0;
@@ -515,19 +562,19 @@ static int common_path_parser(search_items *s, char *path)
 {
 	char *term;
 
+	if (!s->filename) {
+		//create
+		s->filename = malloc(sizeof(slist));
+		if (s->filename == NULL)
+			return 4;
+		slist_create(s->filename);
+	}
 	if (*path == '"') {
 		/* string is normal */
 		path++;
 		term = strchr(path, '"');
 		if (term == NULL)
 			return 1;
-		if (!s->filename) {
-			//create
-			s->filename = malloc(sizeof(slist));
-			if (s->filename == NULL)
-				return 2;
-			slist_create(s->filename);
-		}
 		*term = 0;
 		if (s->filename) {
 			// append
@@ -550,21 +597,21 @@ static int common_path_parser(search_items *s, char *path)
 		}
 		*term = '"';
 	} else { 
-		if (!s->filename) {
-			//create
-			s->filename = malloc(sizeof(slist));
-			if (s->filename == NULL)
-				return 4;
-			slist_create(s->filename);
-		}
 		if (s->filename) {
 			// append
 			snode sn;
-			if (!isxdigit(path[0]))
-				return 5;
-			sn.str = unescape(path);
 			sn.key = NULL;
 			sn.hits = 1;
+			if (strcmp(path, "(null)")) {
+				sn.str = strdup(path);
+				goto append;
+			}
+			if (!isxdigit(path[0]))
+				return 5;
+			if (path[0] == '0' && path[1] == '0')
+				sn.str = unescape(&path[2]); // Abstract name
+			else
+				sn.str = unescape(path);
 			// Attempt to rebuild path if relative
 			if ((sn.str[0] == '.') && ((sn.str[1] == '.') ||
 				(sn.str[1] == '/')) && s->cwd) {
@@ -576,18 +623,20 @@ static int common_path_parser(search_items *s, char *path)
 				free(sn.str);
 				sn.str = tmp;
 			}
+append:
 			slist_append(s->filename, &sn);
 		}
 	}
 	return 0;
 }
 
+/* Older AVCs have path separate from the AVC record */
 static int avc_parse_path(const lnode *n, search_items *s)
 {
 	char *str;
 
 	if (event_filename) {
-	// dont do this search unless needed
+		// dont do this search unless needed
 		str = strstr(n->message, " path=");
 		if (str) {
 			str += 6;
@@ -608,9 +657,11 @@ static int parse_path(const lnode *n, search_items *s)
 		// dont do this search unless needed
 		str = strstr(term, " name=");
 		if (str) {
+			int rc;
 			str += 6;
-			if (common_path_parser(s, str))
-				return 1;
+			rc = common_path_parser(s, str);
+			if (rc)
+				return rc;
 		}
 	}
 	if (event_object) {
@@ -630,7 +681,7 @@ static int parse_path(const lnode *n, search_items *s)
 				if (term)
 					*term = ' ';
 			} else
-				return 6;
+				return 7;
 		}
 	}
 	return 0;
@@ -1164,6 +1215,17 @@ static int parse_avc(const lnode *n, search_items *s)
 			term = str + 6;
 		}
 	}
+	if (event_filename) {
+		// do we have a path?
+		str = strstr(term, " path=");
+		if (str) {
+			str += 6;
+			rc = common_path_parser(s, str);
+			if (rc)
+				goto err;
+			term += 7;
+		}
+	}
 	if (event_subject) {
 		// scontext
 		str = strstr(term, "scontext=");
@@ -1245,7 +1307,7 @@ static int parse_kernel_anom(const lnode *n, search_items *s)
 		term = n->message;
 
 	// get uid
-	str = strstr(term, "uid=");
+	str = strstr(term, " uid="); // if promiscuous, we start over
 	if (str) {
 		ptr = str + 4;
 		term = strchr(ptr, ' ');
