@@ -1,6 +1,6 @@
 /*
  * ausearch.c - main file for ausearch utility 
- * Copyright 2005-07 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2005-08 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -54,6 +54,7 @@ static int str2event(char *s, event *e);
 static int events_are_equal(event *e1, event *e2);
 
 extern char *user_file;
+extern int force_logs;
 extern int match(llist *l);
 extern void output_record(llist *l);
 
@@ -62,14 +63,8 @@ static int input_is_pipe(void)
 	struct stat st;
 
 	if (fstat(0, &st) == 0) {
-		if (S_ISFIFO(st.st_mode)) {
-			if (user_file) {
-				fprintf(stderr,
-				    "-if option conflicts with stdin method");
-				exit(1);
-			}
+		if (S_ISFIFO(st.st_mode)) 
 			pipe_mode = 1;
-		}
 	}
 	return pipe_mode;
 }
@@ -93,10 +88,12 @@ int main(int argc, char *argv[])
 	set_aumessage_mode(MSG_STDERR, DBG_NO);
 	(void) umask( umask( 077 ) | 027 );
 
-	if (input_is_pipe())
-		rc = process_stdin();
-	else if (user_file)
+	if (user_file)
 		rc = process_file(user_file);
+	else if (force_logs)
+		rc = process_logs();
+	else if (input_is_pipe())
+		rc = process_stdin();
 	else
 		rc = process_logs();
 	ilist_clear(event_type);
@@ -157,6 +154,8 @@ static int process_logs(void)
 			free_config(&config);
 			return ret;
 		}
+		if (just_one && found)
+			break;
 
 		/* Get next log file */
 		num--;
@@ -187,6 +186,10 @@ static int process_log_fd(void)
 		if (match(&entries)) {
 			output_record(&entries);
 			found = 1;
+			if (just_one) {
+				list_clear(&entries);
+				break;
+			}
 		}
 		list_clear(&entries);
 	} while (ret == 0);
