@@ -1,6 +1,6 @@
 # audit.rules parsing and creation.
 #
-# Copyright (C) 2007 Red Hat, Inc.  All rights reserved.
+# Copyright (C) 2007, 2008 Red Hat, Inc.  All rights reserved.
 # This copyrighted material is made available to anyone wishing to use, modify,
 # copy, or redistribute it subject to the terms and conditions of the GNU
 # General Public License v.2.  This program is distributed in the hope that it
@@ -148,6 +148,8 @@ class _FilterKeyFieldType(_StringFieldType):
     def parse_value(string, *unused):
         if len(string) > audit.AUDIT_MAX_KEY_LEN:
             raise ParsingError(_('Key value "%s" too long') % string)
+        if util.is_ids_key(string) and util.parse_ids_key(string) is None:
+            raise ParsingError(_('Invalid IDS key "%s"') % string)
         return string
 
 class _ArchFieldType(FieldType):
@@ -432,6 +434,10 @@ class Rule(object):
             # FIXME: this is to avoid -F msgtype=1 -F msgtype=2 not doing the
             # right thing, but it prevents range expressions from working
             raise ParsingError('Only one field is allowed in "exclude" rules')
+        keys = (field.value for field in self.fields
+                if field.var == audit.AUDIT_FILTERKEY)
+        if len('\x01'.join(keys)) > audit.AUDIT_MAX_KEY_LEN:
+            raise ParsingError('Total key length is too long')
         # FIXME: more checks?
 
     def command_text(self, rules, list, list_name):
@@ -454,7 +460,7 @@ class Rule(object):
             if (f.var != audit.AUDIT_FILTERKEY and
                 (f.var != audit.AUDIT_WATCH or not watch_used)):
                 o.append(f.option_text(self))
-        if list is not rules.exclude_rules:
+        if list is not rules.exclude_rules and list is not rules.user_rules:
             for s in self.syscalls:
                 if s == self.SYSCALLS_ALL:
                     o.append('-S all')
@@ -552,6 +558,8 @@ class AuditRules(object):
                     rule_list = self.entry_rules
                 elif dest == 'exit':
                     rule_list = self.exit_rules
+                elif dest == 'user':
+                    rule_list = self.user_rules
                 elif dest == 'exclude':
                     rule_list = self.exclude_rules
                 else:
@@ -616,7 +624,7 @@ class AuditRules(object):
             elif opt == '-h' or opt == '-i' or opt == '-l':
                 pass # Ignore
             elif opt == '-k':
-                # FIXME: warn about more than one key?
+                # This will be ignored if -k is used with -l or -D.
                 field = Field()
                 field.parse_special(audit.AUDIT_FILTERKEY, arg)
                 rule.fields.append(field)
@@ -688,7 +696,7 @@ class AuditRules(object):
             try:
                 self.__interpret_line(self.__whitespace_re.split(line))
             except ParsingError, e:
-                print >> sys.stderr, "audit.rules:%d: %s" % (line_no, e.message)
+                print >> sys.stderr, "audit.rules:%d: %s" % (line_no, str(e))
 
     def write(self, client):
         '''Write current state to the auditd configuration file using client.
