@@ -1,6 +1,6 @@
 # Rule dialog.
 #
-# Copyright (C) 2007 Red Hat, Inc.  All rights reserved.
+# Copyright (C) 2007, 2008 Red Hat, Inc.  All rights reserved.
 # This copyrighted material is made available to anyone wishing to use, modify,
 # copy, or redistribute it subject to the terms and conditions of the GNU
 # General Public License v.2.  This program is distributed in the hope that it
@@ -26,6 +26,7 @@ import audit_rules
 from dialog_base import DialogBase
 import field_help
 import lists
+from key_list_dialog import KeyListDialog
 import util
 
 __all__ = ('RuleDialog')
@@ -44,13 +45,14 @@ class RuleDialog(DialogBase):
                            'rule_field_down', 'rule_field_op', 'rule_field_up',
                            'rule_field_value', 'rule_field_var',
                            'rule_field_var_info', 'rule_fields',
-                           'rule_key', 'rule_key_hbox', 'rule_key_present',
+                           'rule_keys', 'rule_keys_change', 'rule_keys_hbox',
+                           'rule_keys_present',
                            'rule_never', 'rule_notebook',
                            'rule_syscall_add', 'rule_syscall_delete',
                            'rule_syscall_name', 'rule_syscalls',
                            'rule_syscalls_not_all')
 
-    def __init__(self, excluded_fields, action_always_label,
+    def __init__(self, parent, excluded_fields, action_always_label,
                  action_never_label):
         '''Initialize a rule dialog, removing fields for excluded_fields.
 
@@ -58,7 +60,7 @@ class RuleDialog(DialogBase):
         removed as well.
 
         '''
-        DialogBase.__init__(self, 'rule_dialog',
+        DialogBase.__init__(self, 'rule_dialog', parent,
                             notebook_name = 'rule_notebook')
         self.excluded_fields = excluded_fields
 
@@ -66,11 +68,12 @@ class RuleDialog(DialogBase):
         util.connect_and_run(self.rule_always, 'toggled',
                              self.__rule_always_toggled)
         if audit.AUDIT_FILTERKEY not in self.excluded_fields:
-            util.connect_and_run(self.rule_key_present, 'toggled',
-                                 self.__rule_key_present_toggled)
-            self.rule_key.set_max_length(audit.AUDIT_MAX_KEY_LEN)
+            util.connect_and_run(self.rule_keys_present, 'toggled',
+                                 self.__rule_keys_present_toggled)
+            self.rule_keys_change.connect('clicked',
+                                          self.__rule_keys_change_clicked)
         else:
-            self.rule_key_hbox.destroy()
+            self.rule_keys_hbox.destroy()
         self.rule_never.set_label(action_never_label)
         if audit.AUDIT_ARCH not in self.excluded_fields:
             util.connect_and_run(self.rule_arch_present, 'toggled',
@@ -147,6 +150,7 @@ class RuleDialog(DialogBase):
         self.rule_field_value.set_model(self.field_value_store)
         self.rule_field_value.set_text_column(0)
 
+        self.keys = []
         self.machine_id = None
         self.last_arch_name = None
         if audit.AUDIT_ARCH not in self.excluded_fields:
@@ -169,14 +173,10 @@ class RuleDialog(DialogBase):
         '''Modify dialog controls to reflect rule.'''
         self._radio_set(rule.action, self.__action_map)
         if audit.AUDIT_FILTERKEY not in self.excluded_fields:
-            for f in rule.fields:
-                if f.var == audit.AUDIT_FILTERKEY:
-                    self.rule_key_present.set_active(True)
-                    self.rule_key.set_text(f.value)
-                    break
-            else:
-                self.rule_key_present.set_active(False)
-                self.rule_key.set_text('')
+            self.keys = [f.value for f in rule.fields
+                         if f.var == audit.AUDIT_FILTERKEY]
+            self.rule_keys_present.set_active(len(self.keys) != 0)
+            self.__update_rule_keys()
         if audit.AUDIT_ARCH not in self.excluded_fields:
             for f in rule.fields:
                 if f.var == audit.AUDIT_ARCH:
@@ -218,12 +218,13 @@ class RuleDialog(DialogBase):
 
         del rule.fields[:]
         if (audit.AUDIT_FILTERKEY not in self.excluded_fields and
-            self.rule_key_present.get_active()):
-            f = Field()
-            f.var = audit.AUDIT_FILTERKEY
-            f.op = Field.OP_EQ
-            f.value = self.rule_key.get_text()
-            rule.fields.append(f)
+            self.rule_keys_present.get_active()):
+            for key in self.keys:
+                f = Field()
+                f.var = audit.AUDIT_FILTERKEY
+                f.op = Field.OP_EQ
+                f.value = key
+                rule.fields.append(f)
         if audit.AUDIT_ARCH not in self.excluded_fields:
             if self.rule_arch_present.get_active():
                 f = Field()
@@ -264,7 +265,7 @@ class RuleDialog(DialogBase):
             try:
                 util.parse_elf(self.rule_arch.child.get_text())
             except util.ParsingError, e:
-                return (e.message, 0, self.rule_arch)
+                return (str(e), 0, self.rule_arch)
         if (audit.AUDIT_ARCH not in self.excluded_fields and
             self.rule_syscalls_not_all.get_active()):
             it = self.syscall_store.get_iter_first()
@@ -274,13 +275,20 @@ class RuleDialog(DialogBase):
                     util.parse_syscall(name, self.machine_id)
                 except util.ParsingError, e:
                     self.syscalls_selection.select_iter(it)
-                    return (e.message, 1, self.rule_syscalls)
+                    return (str(e), 1, self.rule_syscalls)
                 it = self.syscall_store.iter_next(it)
             if self.syscall_store.get_iter_first() is None:
                 return (_('The system call list must not be empty'), 1,
                         self.rule_syscalls)
         return None
 
+    def __update_rule_keys(self):
+        '''Update text in self.rule_keys'''
+        if len(self.keys) != 0:
+            self.rule_keys.set_text(util.keys_string(self.keys))
+        else:
+            self.rule_keys.set_markup('<i>%s</i>' % _('No key'))
+
     def __update_fields_store_row(self, it):
         '''Update the text in the self.fields_store row selected by it.'''
         field = self.fields_store.get_value(it, 0)
@@ -289,14 +297,23 @@ class RuleDialog(DialogBase):
     def __rule_always_toggled(self, *_):
         if audit.AUDIT_FILTERKEY not in self.excluded_fields:
             util.set_sensitive_all(self.rule_always.get_active(),
-                                   self.rule_key_present,
-                                   # self.rule_key excluded
+                                   self.rule_keys_present,
+                                   # self.rule_keys, self.rule_keys_change
+                                   # excluded
                                    )
-            self.__rule_key_present_toggled()
+            self.__rule_keys_present_toggled()
 
-    def __rule_key_present_toggled(self, *_):
-        self.rule_key.set_sensitive(self.rule_always.get_active() and
-                                    self.rule_key_present.get_active())
+    def __rule_keys_present_toggled(self, *_):
+        util.set_sensitive_all(self.rule_always.get_active() and
+                               self.rule_keys_present.get_active(),
+                               self.rule_keys, self.rule_keys_change)
+
+    def __rule_keys_change_clicked(self, *_):
+        dlg = KeyListDialog(self.window)
+        res = dlg.run(self.keys)
+        dlg.destroy()
+        if res == gtk.RESPONSE_OK:
+            self.__update_rule_keys()
 
     def __rule_arch_present_toggled(self, *_):
         self.rule_arch.set_sensitive(self.rule_arch_present.get_active())
@@ -318,7 +335,7 @@ class RuleDialog(DialogBase):
         try:
             util.parse_syscall(name, self.machine_id)
         except util.ParsingError, e:
-            self._modal_error_dialog(e.message)
+            self._modal_error_dialog(str(e))
             self.rule_syscall_name.grab_focus()
             return
         it = self.syscall_store.get_iter_first()
@@ -340,7 +357,7 @@ class RuleDialog(DialogBase):
             # Changing the focus within a focus change callback looks evil,
             # defer it until it is safe.
             def callback():
-                self._modal_error_dialog(e.message)
+                self._modal_error_dialog(str(e))
                 self.rule_notebook.set_current_page(0)
                 self.rule_arch.child.grab_focus()
                 return False
@@ -413,7 +430,7 @@ class RuleDialog(DialogBase):
             field.parse_triple(var, op, self.rule_field_value.child.get_text())
         except util.ParsingError, e:
             self.rule_notebook.set_current_page(self.fields_page)
-            self._modal_error_dialog(e.message)
+            self._modal_error_dialog(str(e))
             # Guess which widget is incorrect
             self.rule_field_value.child.grab_focus()
             return
