@@ -29,13 +29,35 @@
 #include "auparse.h"
 #include "aulastlog-llist.h"
 
-
-int main(void)
+void usage(void)
 {
+	fprintf(stderr, "usage: aulastlog [--user name]\n");
+}
+
+int main(int argc, char *argv[])
+{
+	int i;
+	char *user = NULL;
 	struct passwd *p;
         auparse_state_t *au;
 	llist l;
 
+	for (i=1; i<argc; i++) {
+		if ((strcmp(argv[i], "--user") == 0) || 
+				(strcmp(argv[i], "-u") == 0)) {
+			i++;
+			if (i<argc)
+				user = argv[i];
+			else {
+				usage();
+				return 1;
+			}
+		} else {
+			usage();
+			return 1;
+		}
+	}
+
         setlocale (LC_ALL, "");
 	list_create(&l);
 
@@ -48,10 +70,18 @@ int main(void)
 		n.name = p->pw_name;
 		n.host = NULL;
 		n.term = NULL;
-		list_append(&l, &n);
+		if (user == NULL)
+			list_append(&l, &n);
+		else if (strcmp(user, p->pw_name) == 0)
+			list_append(&l, &n);
 	}
 	endpwent();
 
+	if (user && list_get_cnt(&l) == 0) {
+		printf("Unknown User: %s\n", user);
+		return 1;
+	}
+
 	// Search for successful user logins
 	au = auparse_init(AUSOURCE_LOGS, NULL);
 	if (au == NULL) {
